@@ -1,7 +1,8 @@
 #' @title Freeman's theta
 #'
 #' @description Calculates Freeman's theta for a table with one ordinal
-#'              variable and one nominal variable.
+#'              variable and one nominal variable; confidence intervals
+#'              by bootstap.
 #' 
 #' @param x Either a two-way table or a two-way matrix.
 #'          Can also be a vector of observations of an ordinal variable.
@@ -14,6 +15,15 @@
 #'                   comparison.
 #' @param progress If \code{TRUE}, prints a message as each comparison is
 #'                 conducted.
+#' @param ci If \code{TRUE}, returns confidence intervals by bootstrap.
+#'           May be slow.
+#' @param conf The level for the confidence interval.
+#' @param type The type of confidence interval to use.
+#'             Can be any of "\code{norm}", "\code{basic}", 
+#'                           "\code{perc}", or "\code{bca}".
+#'             Passed to \code{boot.ci}.
+#' @param R The number of replications to use for bootstrap.
+#' @param histogram If \code{TRUE}, produces a histogram of bootstrapped values.
 #' @param digits The number of significant digits in the output.
 #' 
 #' @details  Freeman's coefficent of differentiation (theta)
@@ -25,6 +35,12 @@
 #'           Currently, the function makes no provisions for \code{NA}
 #'           values in the data.  It is recommended that \code{NA}s be removed
 #'           beforehand.
+#'
+#'           When theta is close to 0 or very large,
+#'           or with small counts in some cells,
+#'           the confidence intervals 
+#'           determined by this
+#'           method may not be reliable, or the procedure may fail.
 #'           
 #' @author Salvatore Mangiafico, \email{mangiafico@njaes.rutgers.edu}
 #' @references Freeman, L.C. 1965. Elementary Applied Statistics for Students
@@ -33,7 +49,9 @@
 #'             \url{http://rcompanion.org/handbook/H_11.html}
 #' @seealso \code{\link{epsilonSquared}}             
 #' @concept correlation Freeman theta ordinal nominal
-#' @return A single statistic, Freeman's theta
+#' @return A single statistic, Freeman's theta.
+#'         Or a small data frame consisting of Freeman's theta,
+#'         and the lower and upper confidence limits. 
 #'         
 #' @examples
 #' data(Breakfast)
@@ -60,11 +78,15 @@
 #' XT = xtabs( ~ Speaker + Likert , data = PoohPiglet)
 #' freemanTheta(XT)
 #' 
+#' @importFrom boot boot boot.ci
+#' 
 #' @export
 
 freemanTheta = function (x, g=NULL, group="row", 
                          verbose=FALSE, progress=FALSE,
-                         digits=3){
+                         ci=FALSE, conf=0.95, 
+                         type="perc",
+                         R=1000, histogram=FALSE, digits=3){
   
   if(is.matrix(x)){x=as.table(x)}
   
@@ -82,7 +104,7 @@ freemanTheta = function (x, g=NULL, group="row",
      }
   }
   g = factor(g)
-  g=droplevels(g)
+  g = droplevels(g)
   k     = length(levels(g))
   Delta = 0
   Tee   = 0
@@ -124,7 +146,56 @@ freemanTheta = function (x, g=NULL, group="row",
   
   theta = Delta / Tee
   Theta = signif(theta, digits=digits)
-  names(Theta)="Freeman.theta"
-
-  return(Theta)
+  
+if(ci==TRUE){
+  Function = function(input, index){
+                      Input = input[index,]
+  Input$g = factor(Input$g)
+  Input$g = droplevels(Input$g)
+  k     = length(levels(Input$g))
+  Delta = 0
+  Tee   = 0
+  Count = 0
+  Di    = rep(NA, ((k-1)*(k-2)))
+  Ti    = rep(NA, ((k-1)*(k-2)))
+  for(i in 1:(k-1)){
+     for(j in (i+1):k){
+       Count = Count + 1
+       Y1 = Input$x[Input$g==levels(Input$g)[i]]
+       Y2 = Input$x[Input$g==levels(Input$g)[j]]
+       n1 = length(Y1)
+       n2 = length(Y2)
+       tee   = 0
+       delta = 0
+       for(l in 1:n1){
+         for(m in 1:n2){
+           delta = delta + sum(Y1[l] > Y2[m]) - sum(Y1[l] < Y2[m])
+         }}
+       Di[Count] = abs(delta)
+       Ti[Count] = n1 * n2
+       Delta = Delta + Di[Count]
+       Tee = Tee + Ti[Count]
+       
+     }
+  }
+    return(Delta / Tee)
+  }
+  Data = data.frame(x,g)
+  Boot = boot(Data, Function, R=R)
+  BCI  = boot.ci(Boot, conf=conf, type=type)
+  if(type=="norm") {CI1=BCI$normal[2];  CI2=BCI$normal[3];}
+  if(type=="basic"){CI1=BCI$basic[4];   CI2=BCI$basic[5];}
+  if(type=="perc") {CI1=BCI$percent[4]; CI2=BCI$percent[5];}
+  if(type=="bca")  {CI1=BCI$bca[4];     CI2=BCI$bca[5];}  
+  
+  CI1=signif(CI1, digits=digits)
+  CI2=signif(CI2, digits=digits)
+  
+  if(histogram==TRUE){hist(Boot$t[,1], col = "darkgray")}
+  
+}
+  
+if(ci==FALSE){names(Theta)="Freeman.theta"; return(Theta)}
+if(ci==TRUE){names(Theta) = ""
+             return(data.frame(Freeman.theta=Theta, lower.ci=CI1, upper.ci=CI2))}
 }
